import { Collection, Entity, Enum, ManyToOne, OneToMany, PrimaryKey, Property } from '@mikro-orm/core'
import { BaseEntity } from '#application/bases/baseEntity'
import { Character } from './character'
import { Sprite } from './sprite'
import { CharacterGender, CharacterRace } from '#application/enums'

@Entity()
export class CharacterType extends BaseEntity {
  @PrimaryKey()
  id!: number

  @Property()
  name!: string

  @Enum(() => CharacterGender)
  gender!: CharacterGender

  @Enum(() => CharacterRace)
  race!: CharacterRace

  @Property()
  isSelectable = false

  @OneToMany(() => Character, (character) => character.characterType)
  characters = new Collection<Character>(this)

  @ManyToOne(() => Sprite, { nullable: true })
  sprite?: Sprite

  @Property()
  createdAt = new Date()

  @Property()
  updatedAt = new Date()
}
