import { ExtendedCharacter, TSocket } from '#application/types'
import { Server } from 'socket.io'
import ZoneManager from '#managers/zoneManager'
import { gameLogger } from '#application/logger'
import { ZoneEventTileTeleport } from '#entities/zoneEventTileTeleport'

export class ZoneEventTileService {
  public async handleTeleport(io: Server, socket: TSocket, character: ExtendedCharacter, teleport: ZoneEventTileTeleport): Promise<void> {
    if (teleport.toZone.id === character.zone.id) return

    const loadedZone = ZoneManager.getZoneById(teleport.toZone.id)
    if (!loadedZone) {
      gameLogger.error('zone:character:join error', 'Loaded zone not found')
      return
    }

    const zone = loadedZone.getZone()

    const oldZoneId = character.zone.id
    const newZoneId = teleport.toZone.id

    // Update local character object
    character.zone = teleport.toZone
    character.rotation = teleport.toRotation
    character.positionX = teleport.toPositionX
    character.positionY = teleport.toPositionY
    character.isMoving = false

    await character.save()

    // Remove and add character to new zone
    await loadedZone.removeCharacter(character.id)
    loadedZone.addCharacter(character)

    // Emit events
    io.to(oldZoneId.toString()).emit('zone:character:leave', character.id)
    io.to(newZoneId.toString()).emit('zone:character:join', character)

    // Update socket rooms
    socket.leave(oldZoneId.toString())
    socket.join(newZoneId.toString())

    // Send teleport information to the client
    socket.emit('zone:character:teleport', {
      zone,
      characters: loadedZone.getCharactersInZone()
    })
  }
}
