import { Server } from 'socket.io'
import { TSocket } from '#application/types'
import prisma from '#application/prisma'
import characterRepository from '#repositories/characterRepository'
import { CharacterGender, CharacterRace } from '@prisma/client'
import { gameMasterLogger } from '#application/logger'

type Payload = {
  id: number
  name: string
  gender: CharacterGender
  isSelectable: boolean
  spriteId: string
}

export default class CharacterHairUpdateEvent {
  constructor(
    private readonly io: Server,
    private readonly socket: TSocket
  ) {}

  public listen(): void {
    this.socket.on('gm:characterHair:update', this.handleObjectUpdate.bind(this))
  }

  private async handleObjectUpdate(data: Payload, callback: (success: boolean) => void): Promise<void> {
    const character = await characterRepository.getById(this.socket.characterId as number)
    if (!character) return callback(false)

    if (character.role !== 'gm') {
      return callback(false)
    }

    try {
      await prisma.characterHair.update({
        where: { id: data.id },
        data: {
          name: data.name,
          gender: data.gender,
          isSelectable: data.isSelectable,
          spriteId: data.spriteId
        }
      })

      return callback(true)
    } catch (error) {
      gameMasterLogger.error(`Error updating character hair: ${error instanceof Error ? error.message : String(error)}`)
      return callback(false)
    }
  }
}
